import * as Mongo from 'mongodb'
import { MqttMessage } from './mqtt_message'
import { Instance } from './instance'


export default class MongoBackend extends Instance {
  constructor(options : any) {
    super(options)
    this.on('reconnectDatabase', this.connectToDatabase)
  }
  
  private dbHandle : Mongo.Db;
  private dbReady : boolean = false;
  
  connectToDatabase() {
    this.loginfo("About to connect to database")
    Mongo.MongoClient.connect(this.options.mongodbUrl) 
    .then(
      (tmpDbHandle: Mongo.Db) => { 
        this.dbHandle = tmpDbHandle
        this.dbReady = true; 
        this.loginfo(`Database ${this.options.mongodbUrl} connected`)
        this.dbHandle.on('reconnectFailed', (err : any) => { this.logwarn(`Error on database ${err}`) })
        this.dbHandle.on('reconnect', () => { 
          this.loginfo("Reconnect on database") 
          this.dbReady = true
        })
        this.dbHandle.on('timeout', () => { this.logwarn("Timeout on database") })
        this.dbHandle.on('close', () => { 
          this.loginfo("Close on database") 
          this.dbReady = false
        })
      },
      (err: String) => {
        this.logerror(`Unable to connect to database: ${err}`)
        this.dbReady = false
      }
    )
  }
  
  connectToQueue() {
    this.on('data', (msg : MqttMessage) => {
      if (this.dbReady) { 
        let coll = this.dbHandle.collection(this.options.collection)
        coll.insertOne(msg.getMessage())
        .then(
          (r) => { 
            if (this.options.verbose) {
              this.loginfo(`Successfully inserted into database ${this.options.mongodbUrl}, ${this.options.collection}: ${JSON.stringify(msg.getMessage())}`)
            }
          },
          (err) => { 
            this.dbReady = false
            this.logerror(`Error when trying to insert into database ${err}`)
            this.emit('reconnectDatabase')
          }
        )
      }
    })
  }

/*    
        }
      } else {
        // this.loginfo("Database currently not available, not reading from stream")
      }
    })
    
  }
  */
}
